using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Windows.Forms.Diagram.Controls;

namespace Arc_ArcTool
{
	/// <summary>
	/// Summary description for MyArcTool.
	/// </summary>
	public class MyArcTool : ArcTool,IMouseEventReceiver
	{
		private ShapeFactory shapeFactory = null;
		private MyArcTool(): base(Resources.Strings.Toolnames.Get("ArcTool"))
		{
			this.shapeFactory = new ShapeFactory(this.DefaultFactory);
		}

		public MyArcTool(string name) : base(name)
		{
			this.shapeFactory = new ShapeFactory(this.DefaultFactory);
		}
		public ShapeFactory ShapeFactory
		{
			set
			{
				this.shapeFactory = value;
			}
		}
		protected override void OnActivate()
		{
			this.ChangeCursor(Cursors.Cross);
		}
	protected virtual void OnArcComplete()
		{
			PointF[] worldPts = new PointF[2];
			worldPts[0] = this.Controller.View.ViewToWorld(this.Controller.View.DeviceToView(this.startingPt));
			worldPts[1] = this.Controller.View.ViewToWorld(this.Controller.View.DeviceToView(this.endingPt));

			if((this.shapeFactory != null) && (worldPts[0] != worldPts[1]))
			{
				Shape shape = this.shapeFactory(worldPts);
				if (shape != null)
				{
					InsertNodesCmd cmd = new InsertNodesCmd();
					cmd.Nodes.Add(shape);

					try
					{
						this.Controller.ExecuteCommand(cmd, false);
						this.View.SelectionList.Concat(cmd.Nodes);
						this.Controller.UpdateAllViews();
					}
					catch (Exception ex)
					{
						this.Controller.DeactivateTool(this);
						this.View.Refresh();
						throw ex;
					}	
				}
			}
		}
		protected Shape DefaultFactory(PointF[] pts)
		{
			return new Syncfusion.Windows.Forms.Diagram.Arc(pts);
		}
		#region IMouseEventReceiver Members

		public void MouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			if(this.Active)
			{
				if (e.Button == MouseButtons.Left)
				{
					this.startingPt = this.Controller.View.SnapPointToGrid(e.X, e.Y);
					this.drawing = true;
				}
			}
		}

		public void MouseMove(System.Windows.Forms.MouseEventArgs e)
		{
			if (this.Active && this.drawing)
			{
				System.Drawing.Point pt = this.Controller.View.SnapPointToGrid(e.X, e.Y);
				System.Drawing.Rectangle rcBounds;
				float startAngle;
				float sweepAngle;
				Geometry.ArcFromPoints(this.startingPt, pt, out rcBounds, out startAngle, out sweepAngle);
				//Controller.View.DrawTrackingRect(rcBounds);
				this.Controller.View.Refresh();
				this.Controller.View.DrawTrackingArc(rcBounds, startAngle, sweepAngle);
			}
		}

		public void MouseUp(System.Windows.Forms.MouseEventArgs e)
		{
			if(this.Active && this.drawing)
			{
				this.endingPt = this.Controller.View.SnapPointToGrid(e.X, e.Y);
				this.drawing = false;
				this.OnArcComplete();
			}
		}

		#endregion
	}
	
}
